# Plone Solutions AS <info@plonesolutions.com>
# Martijn Pieters <mj@plonesolutions.com>
# http://www.plonesolutions.com

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

from zope.interface import Attribute

from Products.PluginIndexes.interfaces import IPluggableIndex

class ILanguageIndex(IPluggableIndex):
    """Index for ITranslatable Language tags
    
    The index treats language tags according to RFC 1766, with the assumption
    that languages with more than one subtag but with the same initial subtag 
    as related; en-us is related to en-gb, and will fall back to related
    content (same canonical UID) when queried.
    
    Language tags are treated as containing a initial subtag, and one optional
    second subtag, splitting a tag on the first dash. Thus, this index does
    not support additional subtags, but modern browsers never send anything 
    more complicated than a main language with a region or dialect, nor do
    current Plone i18n tools support anything else.
    
    Fallback rules are as follows: if searching for a given language tag, a 
    union is returned of:
    
    1. Exact tag match
    2. Initial subtag match, no second subtag (if not the original search)
    3. Any other second subtags indexed for the initial subtag in alfabetical
       order.
    
    Results are filtered for unique canonical UIDs, thus ensuring only one
    translation per content item is returned.
    
    So, when searching for en-gb, all matches for en-gb are returned, plus
    any matches for en, en-au, en-ca, en-nz, en-us if such tags were indexed
    and not already translated into en-gb.
    
    The fallback behaviour can be disabled by setting index.fallback to False,
    or passing a 'fallback' query parameter (also a boolean) to the index
    when querying.

    """
    
    fallback = Attribute(
        'fallback',
        """Wether or not to enable language fallback querying.
        
        Boolean, defaults to True.
        """,
        )
